// DTSpyView.cpp : implementation of the CDTSpyView class
//

#include "stdafx.h"
#include "DTSpy.h"

#include "DTSpyDoc.h"
//#include "diting.h"
#include "..\mfcditing\mfcdt10.h"

// The following is to disable the long identifer string warning in debug mood. This is cause 
// by our use of templates std::map with std::string. I tried including #pragma warning(disable:4786) 
// in header DTSpyView.h but it didn't work. I guess this #pragma can only be used in .c file?

#ifdef _DEBUG
#pragma warning(disable:4786)
#endif

#include "DTSpyView.h"
#include "config.h"
#include "AddAppDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


typedef struct {
	TCHAR cmd[MAX_PATH];           // Command line
	TCHAR currentDir[MAX_PATH];
	BOOL needURL; 
	// Retrieve URL from child window text or not.

	// other info?
} myAppInfo;

//#define MAX_REGISTERED_APPS   64
static myAppInfo g_registeredApps[MAX_REGISTERED_APPS];
static int g_nRegisteredApps = 0;



/////////////////////////////////////////////////////////////////////////////
// CDocInfo and CAppInfo


CDocInfo::CDocInfo(const TCHAR *theTitle, const TCHAR *theUser, CAppInfo * theApp) {
	if( theTitle != NULL ) {
        if(lstrlen(theTitle) < MAX_PATH && lstrlen(theUser) < MAX_PATH) {
			lstrcpy(title, theTitle);
            lstrcpy(user, theUser);
        }
	}
	pApp = theApp;
	elapsed = 0;
}


void CDocInfo::Write(ostream os) {
	os<<user<<"\t"<<title<<": "<<elapsed<<endl;
}


CAppInfo::CAppInfo(const TCHAR *theCmd, int id, bool url) {
	if( theCmd != NULL ) {
		if(lstrlen(theCmd) < MAX_PATH)
			lstrcpy(cmd, theCmd);
	}
	ID = id;
	needURL = url;
}

CAppInfo::~CAppInfo() {
	CDocInfo * pDocInfo = NULL;

	for (DocMap::iterator dm = docMap.begin(); dm != docMap.end();dm++) {
		pDocInfo = (*dm).second;
		if(pDocInfo != NULL)
			delete pDocInfo;
	}

	nDocs = 0;
}


CDocInfo * CAppInfo::AddDoc(const TCHAR *theTitle, const TCHAR *theUser) {
	string title = theTitle;
	CDocInfo *pDI = new CDocInfo(theTitle, theUser, this);
    
	if(pDI != NULL) {
		docMap.insert(DocMap::value_type(title, pDI));
		nDocs++;
	}
	return pDI;
}

void CAppInfo::Write(ostream os) {
	os<<"["<<cmd<<"]"<<endl;
	for (DocMap::iterator dm = docMap.begin(); dm != docMap.end();dm++) {
		CDocInfo * pDocInfo = (*dm).second;
		if(pDocInfo != NULL)
			pDocInfo->Write(os);
	}
	os<<endl;
}
   

/////////////////////////////////////////////////////////////////////////////
// CDTSpyView

IMPLEMENT_DYNCREATE(CDTSpyView, CView)

BEGIN_MESSAGE_MAP(CDTSpyView, CView)
	//{{AFX_MSG_MAP(CDTSpyView)
	ON_WM_COPYDATA()
	ON_WM_CREATE()
	ON_COMMAND(ID_ADDAPP, OnAddapp)
	ON_WM_TIMER()
	ON_WM_DESTROY()
    ON_MESSAGE(WM_MYNOTIFYICON, OnMynotifyicon)
	ON_MESSAGE(WM_PROCESSDATA, OnProcessdata)
	ON_COMMAND(ID_APP_EXIT, OnAppExit)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()


extern CDTSpyApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CDTSpyView construction/destruction

CDTSpyView::CDTSpyView()
{
	m_cx = m_cy = 8;
	m_MSG = "HELLO";
//	m_fileName = "g:\\research\\abc\\logging\\out.txt";
	m_fileName = "out.txt";
	m_outFile.open(m_fileName, ios_base::out | ios_base::app);
//    m_outFile << "Time          Application                     Doc                      Action" << endl;
//    m_outFile << "-----------------------------------------------------------------------------" << endl;

	pDocPrev = NULL;
	t_prev = 0;
    m_nTicks = 0;
	appID = ID_NOTREGISTERED;
	nCode = -1;
}

CDTSpyView::~CDTSpyView()
{
	RemoveHook();
//    KillTimer(m_IDTimer);
    
    // Finish the last doc:
    if(pDocPrev != NULL) {
        time_t now = time(NULL);     // Get UNIX-style time
        char * timestr = ctime(&now);
        timestr[lstrlen(timestr)-1] = '\0'; // Strip off the newline

        m_outFile<<"\""<<timestr<<"\""<<"\t" << pDocPrev->user <<"\t" << pDocPrev->pApp->cmd <<"\t" << pDocPrev->title <<"\t\"Stop logging\""<<endl;
    }
    m_outFile.close();

	CAppInfo * pAppInfo = NULL;
	ofstream logFile;
	time_t now = time(NULL);

	if(pDocPrev != NULL) 
		pDocPrev->elapsed += (now - t_prev);

	logFile.open("log.txt");
	for (AppMap::iterator am = appMap.begin(); am != appMap.end(); am++) {
		pAppInfo = (*am).second;
		if(pAppInfo != NULL){
			pAppInfo->Write(logFile);
			delete pAppInfo;
		}
	}
	logFile.close();
}

BOOL CDTSpyView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDTSpyView drawing

void CDTSpyView::OnDraw(CDC* pDC)
{
	CDTSpyDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CSize size = pDC->GetTextExtent(m_MSG);
	m_cy += size.cy;
	pDC->TextOut(m_cx, m_cy, m_MSG);
}

/////////////////////////////////////////////////////////////////////////////
// CDTSpyView printing

BOOL CDTSpyView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDTSpyView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDTSpyView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CDTSpyView diagnostics

#ifdef _DEBUG
void CDTSpyView::AssertValid() const
{
	CView::AssertValid();
}

void CDTSpyView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CDTSpyDoc* CDTSpyView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDTSpyDoc)));
	return (CDTSpyDoc*)m_pDocument;
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////

/** 
 * Parse the data passed in the CopyData message and fill in the command 
 * document title. szDoc will be the URL if url == true.
 *
 * The message is formated as 
 *
 *   Cmd:"*****" ["***"], Caption:*****, Dir:******, URL:******, User:*****
 *
 * returns TRUE if data processed successfully, FALSE otherwise.
 *
 */
BOOL CDTSpyView::ParseMessgeData(const TCHAR * szBuffer, TCHAR * szCmd, 
								 TCHAR * szDoc, TCHAR * szUser, bool url) {
	TCHAR *szDir = NULL;
	TCHAR *szCaption = NULL;
	TCHAR *szCmdWithArgs = NULL;
	// The cmd passed in CopyData message is obtained by GetCommandLine()
	// and may include arguments.
	TCHAR *szURL = NULL;
    TCHAR *sztmpUser = NULL;
	TCHAR *szArgs = NULL;
    TCHAR buf[1024];
    TCHAR szCmdBuf[1024];
    TCHAR szDocBuf[1024];


    ZeroMemory(buf, 1024);
    ZeroMemory(szCmdBuf, 1024);
    ZeroMemory(szDocBuf, 1024);
    
	szCmdWithArgs = strstr(szBuffer, "Cmd:");
	szCaption = strstr(szBuffer, "Caption:");
	szDir = strstr(szBuffer, "Dir:");
	szURL = strstr(szBuffer, "URL:");
    sztmpUser = strstr(szBuffer, "User:");

//	m_outFile<<"In ParseMessgeData::Pointers = "<<(int)szCmdWithArgs<<", "<<(int)szCaption<<", "
//		<<(int)szDir<<", "<<(int)szURL<<", "<<(int)sztmpUser<<endl;

	if(szCmdWithArgs == NULL || szCaption == NULL ||
		szDir == NULL || szURL == NULL  || sztmpUser == NULL)
		return FALSE;

	// Point them to the actual string:
	szCmdWithArgs += lstrlen("Cmd:");
	szCaption += lstrlen("Caption:");
	szDir += lstrlen("Dir:");
	szURL += lstrlen("URL:");
    sztmpUser += lstrlen("User:");


	*(szCaption - lstrlen("Caption:") - 2) = '\0';  // Terminate szCmdWithArgs
	*(szDir - lstrlen("Dir:") - 2) = '\0';  // Terminate szCaption;
    *(szURL - lstrlen("URL:") - 2) = '\0';  // Terminate szDir;
    *(sztmpUser - lstrlen("User:") - 2) = '\0';  // Terminate szURL;

    CopyMemory(szUser, sztmpUser, lstrlen(sztmpUser));
    szArgs = strchr(szCmdWithArgs + 1, '\"');
	// Command is enclosed in " ... ", followed by arguments. But it can be NULL!

	if(szArgs != NULL) {
        CopyMemory(szCmdBuf, szCmdWithArgs, szArgs - szCmdWithArgs + 1);
        //CopyMemory(szCmd, szCmdWithArgs, szArgs - szCmdWithArgs + 1);
		szArgs += 2;
	}
    else 
        CopyMemory(szCmdBuf, szCmdWithArgs, szCaption - szCmdWithArgs - (lstrlen("Caption:")+2));

    FixBackSlash(szCmdBuf, szCmd);


    // Now lets try to fix the arguments, Anything preceeded by / or - will be considered
    // options and the rest will be the file name.
/*    if(szArgs != NULL) {
        char *p = szArgs;
        while(*p == '/' || *p == '-') {
            p = strchr(p, ' ');   // Find the next space.
            if(p == NULL)
                break;
            p++;    // Point to the char right after the space
        }
        szArgs = p;
    }
*/

    CopyMemory(buf, szCmdWithArgs, lstrlen(szCmdWithArgs));
    if(strstr(_strlwr(buf), "word") || strstr(_strlwr(buf), "excel")
        || strstr(_strlwr(buf), "powerp") || strstr(_strlwr(buf), "msaccess")) {
        // For office apps the full path of the current doc is in szDir
		// Add quote to the doc full path:
		sprintf(buf, "\"%s\"", szDir);
   		CopyMemory(szDocBuf, buf, sizeof(TCHAR)*strlen(buf));
		//CopyMemory(szDoc, szDir, sizeof(TCHAR)*lstrlen(szDir));
        FixBackSlash(szDocBuf, szDoc);
        return TRUE;
    }
   
   
    if(url) {  // Fill szDoc with URL
		sprintf(buf, "\"%s\"", szURL);
		CopyMemory(szDocBuf, buf, lstrlen(buf));
    }
	else { // Doc title can be part of caption or command.
		*(szURL - lstrlen("URL:") - 2) = '\0';  // Terminate szDir;
		*(szDir - lstrlen("Dir:") - 2) = '\0';  // Terminate szCaption;
		//if(strstr(szCmdWithArgs, "WINWORD") != NULL || strstr(szCmdWithArgs, "NOTEPAD") != NULL) { 
		if(strstr(szCmdWithArgs, "NOTEPAD") != NULL || strstr(szCmdWithArgs, "notepad") != NULL) { 
			// These application's captions are like "docname - appname". So extract the doc
			// name from the caption and append to dir to get the full path of a doc:
			TCHAR *p = strrchr(szCaption, '-');   // Find the last occurrence of '-' 

			if( p != NULL)
				*(p-1) = '\0';

			if(lstrlen(szCaption) > 0)
				wsprintf(szDocBuf, "\"%s\\%s\"", szDir, szCaption);
			else
				*szDocBuf = '\0';
		}
		else if(strstr(szCmdWithArgs, "Acrobat") != NULL 
			|| strstr(szCmdWithArgs, "acrobat") != NULL
			|| strstr(szCmdWithArgs, "ACROBAT") != NULL ) { 
			// Acrobat's command is like "command" "docpath" with quotation??
            if(szArgs != NULL) {
                szArgs = strchr(szArgs, '\"');
                if(szArgs != NULL)
                    CopyMemory(szDocBuf, szArgs, sizeof(TCHAR)*strlen(szArgs));
                else
                    CopyMemory(szDocBuf, "\"null\"", sizeof(TCHAR)*strlen("\"null\""));
            }
            else
                CopyMemory(szDocBuf, "\"null\"", sizeof(TCHAR)*strlen("\"null\""));


//            FixBackSlash(szDocBuf, szDoc);
//            return TRUE;
			//CopyMemory(szDoc, "hello here", sizeof(TCHAR)*strlen("hello here"));
			//sscanf(szCmdWithArgs, "\"%s\" %s", buf, szDoc);
		}
		else {
			CopyMemory(szDoc, szCaption, szDir - szCaption - (lstrlen("Dir:") + 2));
			sprintf(buf, "\"%s\"", szDoc);
			CopyMemory(szDocBuf, buf, lstrlen(buf));
		}
	}
    
    FixBackSlash(szDocBuf, szDoc);
	return TRUE;
}
   

/**
 * Windows uses backslash \ as separator in a path and the path returned 
 * by some API like GetCommandLine() comtains a single backslash for each
 * separator. This causes trouble for some postprocessor who cannot distiguish
 * a separator from an escape sequence (e.g. \n). This function turns a single
 * backslash \ in to an escape sequence '\\' so the postprocessor can recognize
 * it as a backslash. I.e. "\Program Files\Internet Explorer\IEXPLORE.EXE" is
 * turned into "\\Program Files\\Internet Explorer\\IEXPLORE.EXE". If the path
 * is already using \\, then simply copy src to dest.
 */
 
void CDTSpyView::FixBackSlash(TCHAR * src, TCHAR * dest) {
    char *psrc = src;
    char *pdest = dest;

    while(*psrc) {
        *pdest = *psrc;
        if(*psrc == '\\' && *(psrc+1) == '\\') {
            *(pdest+1) = *(psrc+1);
            pdest += 2;
            psrc += 2;   // Skip existing "\\"
            continue;
        }

        if(*psrc == '\\' && *(psrc+1) != '\\')
            *(++pdest) = '\\';
        pdest++;
        psrc++;
    }
}


/////////////////////////////////////////////////////////////////////////////
// CDTSpyView message handlers
// Note that we don't write the data to log in this function because the 
// message is sent with SendMessage and we don't want to block the sender.
// Instead we copy the data to a memory buffer and do I/O later.

BOOL CDTSpyView::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct) {
	now = time(NULL);            // Get the timestamp
	appID = (int)LOWORD(pCopyDataStruct->dwData);
	nCode = (int)HIWORD(pCopyDataStruct->dwData);
	// Now lets get the text passed in the message:

	DWORD dwSize = BUFFERLEN - 1;
	if(pCopyDataStruct->cbData < dwSize)
		dwSize = pCopyDataStruct->cbData;
	CopyMemory(szBuffer, pCopyDataStruct->lpData, dwSize);

//	m_outFile<<"In OnCopyData::Buffer = "<<szBuffer<<endl;

	// Post a message to ourselves:
	::PostMessage(GetSafeHwnd(), WM_PROCESSDATA, NULL, NULL);
	
	return CView::OnCopyData(pWnd, pCopyDataStruct);
}

void CDTSpyView::OnProcessdata(WPARAM wParam, LPARAM lParam) {
	TCHAR szCmd[MAX_PATH] = {_T('\0')};
	TCHAR szDoc[MAX_PATH] = {_T('\0')};
    TCHAR szUser[MAX_PATH] = {_T('\0')};
	AppMap::iterator theIterator;
	CAppInfo *pAppInfo;

	if(appID == ID_NOTREGISTERED && pDocPrev != NULL) {  // Terminate prev doc
		char * timestr = ctime(&now);
		timestr[lstrlen(timestr)-1] = '\0'; // Strip off the newline
        FixBackSlash(pDocPrev->pApp->cmd, szCmd);
        FixBackSlash(pDocPrev->title, szDoc);
        // Sometimes when you save the string, they turn "\\" into a single slash?

		m_outFile<<"\""<<timestr<<"\""<<"\t" << pDocPrev->user <<"\t" << szCmd <<"\t" << szDoc <<"\t\"INACTIVE\""<<endl;
        pDocPrev = NULL;
            
        if(m_nTicks >= TIME_NATURAL_FLUSH) {
			FlushLog();
			m_nTicks = 0;
		}
		return; 
	}

//m_outFile<<"Received MSG to process data: appID = "<<appID<<endl; 
//m_outFile<<"         Buffer: "<<szBuffer<<endl;

	theIterator = appMap.find(appID);
	if(theIterator != appMap.end()) {
		pAppInfo = (*theIterator).second;
			
		if(pAppInfo != NULL) {

//m_outFile<<"         Found app in my list: "<<pAppInfo->cmd<<endl;
//m_outFile<<"         Its ID: "<<pAppInfo->ID<<", needURL: "<<pAppInfo->needURL<<endl;

			if(!ParseMessgeData(szBuffer, szCmd, szDoc, szUser, pAppInfo->needURL))
				return;

			if(lstrlen(szDoc) > 0) {
				// Lets see if we have seen this doc before:
				DocMap::iterator theDocIterator;
				CDocInfo *pDocInfo;

				theDocIterator = pAppInfo->docMap.find(szDoc);
				if(theDocIterator != pAppInfo->docMap.end()) 
					pDocInfo = (*theDocIterator).second;
				else 
					pDocInfo = pAppInfo->AddDoc(szDoc, szUser);
					// AddDoc will create CDocInfo

				if(pDocInfo != NULL) {
					if(pDocInfo != pDocPrev) { // This is a different doc
						if(pDocPrev == NULL) {
							pDocPrev = pDocInfo;
							time(&t_prev);
						}

						pDocPrev->elapsed += (now - t_prev);
 						pDocPrev = pDocInfo;
						t_prev = now;


                        // Get UNIX-style time
                        char * timestr = ctime(&now);
                        timestr[lstrlen(timestr)-1] = '\0'; // Strip off the newline
						char szCode[64];

						switch (nCode)  { 
						case HCBT_ACTIVATE:
						case HCBT_SETFOCUS: 
							strcpy(szCode, "ACTIVATE"); 
							break; 
						case HCBT_DESTROYWND:
							strcpy(szCode, "CLOSE"); 
							break; 
						default:
							strcpy(szCode, "ACTIVATE"); 
							break; 
						}

                        m_outFile<<"\""<<timestr<<"\""<<"\t"<< szUser <<"\t" << szCmd <<"\t" << szDoc <<"\t"<<"\""<<szCode<<"\""<<endl;

                        // Maybe time to flush the log?
                        if(m_nTicks >= TIME_NATURAL_FLUSH) {
							FlushLog();
                            m_nTicks = 0;
						}
					}
				}
			}
		}
	}
    else if(appID != ID_NOTREGISTERED) {
		char msgbuf[64];
		sprintf(msgbuf, "Notified of application ID: %d, but not found in appMap. ID_NOTREGISTERED is %d", 
			appID, ID_NOTREGISTERED);
        // This should never happen.
		m_outFile<<"         "<<msgbuf<<endl;
//		MessageBox(msgbuf, "DTSpy"); 
	}

//	Invalidate(FALSE);
}

int CDTSpyView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Make it always on top:
	//	SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

	::SetWindowPos(GetSafeHwnd(), HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

    m_IDTimer = SetTimer(1, TIME_OUT, NULL);
    // Setting TimerProc to NULL and we will receive WM_TIMER message.
    // Can't do it in constructor because hwnd


	// Install the hook. Can't do it in constructor because hwnd
	// hasn't initialized yet.
	InitializeDiTingDLL();
    InstallSystemHook(GetSafeHwnd(), WH_CBT, FALSE, NULL, 0);


    // Put itself on the systtray area.
    hIconSysTray = theApp.LoadIcon(IDI_ICONSYSTRAY);
    if(hIconSysTray != NULL) {
        NOTIFYICONDATA tnid; 
        TCHAR * lpszTip = "DT Spy. The nonintrusive logger.";
 
        tnid.cbSize = sizeof(NOTIFYICONDATA); 
        tnid.hWnd = GetSafeHwnd(); 
        tnid.uID = IDI_ICONSYSTRAY; 
        tnid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; 
        tnid.uCallbackMessage = WM_MYNOTIFYICON; 
        tnid.hIcon = hIconSysTray; 
        lstrcpyn(tnid.szTip, lpszTip, sizeof(tnid.szTip)); 

        Shell_NotifyIcon(NIM_ADD, &tnid); 
 
       if(hIconSysTray) 
           DestroyIcon(hIconSysTray); 
 
    }

    // Read from config file 
    Config cfg;
    cfg.Load("dtspy.conf");
    /**
     * Config::Load will terminate the program if the config file
     * is malformated but it won't exit if the file doesn't exist.
     * This case is the same as having an empty config file.
     */
    
    // Try to load config. Use default if fail. 
    char str[64];
    try {
        sprintf(str, "%s", cfg.GetString("SysRoot"));
    }
    catch(CfgException e) {
        str[0] = 'C';   // Default
		str[1] = '\0'; 
    }

	// Register the apps that we are interested:
    CAppInfo * pApp;
	int id;
	const int numApps = 9;
   	TCHAR appBuf[MAX_PATH];
	TCHAR apps[numApps][128] = {
			"\\Program Files\\Internet Explorer\\IEXPLORE.EXE",
			"\\Program Files\\Microsoft Visual Studio\\Common\\MSDev98\\Bin\\MSDEV.EXE",
			"\\Program Files\\Microsoft Office\\Office\\WINWORD.EXE",
			"\\Program Files\\Microsoft Office\\Office\\EXCEL.EXE",
			"\\Program Files\\Microsoft Office\\Office\\POWERPNT.EXE",
			"\\Program Files\\Microsoft Office\\Office\\MSACCESS.EXE",
			"\\Program Files\\Adobe\\Acrobat 5.0\\Acrobat\\Acrobat.exe",
			"\\emacs\\bin\\emacs.exe",
			"\\WINNT\\system32\\NOTEPAD.EXE"
	};
	bool needURL[numApps] = { true, false, false, false, false, false, false, false, false};

	for(int i = 0; i < numApps; i++) {
		sprintf(appBuf, "\"%s:%s\"", str, apps[i]);
		id = RegisterApp(appBuf, needURL[i]);
		if(id >= 0) {
			pApp = new CAppInfo(appBuf, id, needURL[i]);
			appMap.insert(AppMap::value_type(id, pApp));
		}
	}

	return 0;
}

void CDTSpyView::OnAddapp() 
{
	CAppInfo * pApp;	
	CAddAppDlg dlg;
	CString appPath;

	if (dlg.DoModal() == IDOK){
//		m_outFile <<"Adding application, Cmd: " <<(LPCTSTR)dlg.m_appPath<<", needURL: "<<dlg.m_needURL<<endl;

		if(dlg.m_appPath.GetAt(0) != '\"') 
			appPath = '\"' + dlg.m_appPath + '\"';
		else 
			appPath = dlg.m_appPath;

		int id = RegisterApp((LPCTSTR)appPath, dlg.m_needURL != 0);
//		m_outFile <<"RegisterApp returned " << id << endl;

		if(id >= 0) {
			pApp = new CAppInfo((LPCTSTR)appPath, id, dlg.m_needURL != 0);
			appMap.insert(AppMap::value_type(id, pApp));
		}
	}
}

void CDTSpyView::OnTimer(UINT nIDEvent) {
    m_nTicks++;
    if(m_nTicks >= TIME_FORCE_FLUSH) { // It's been too long since we flushed log. 
        FlushLog();
        m_nTicks = 0;;
    }
//    MessageBeep(0xFFFFFFFF);   // Beep
//    m_MSG = "Timer expired.";
//    Invalidate(FALSE);
	CView::OnTimer(nIDEvent);
}

void CDTSpyView::OnDestroy() {
	CView::OnDestroy();
    KillTimer(m_IDTimer);
    // CWnd::KillTimer requires m_hWnd to be valid when this function
    // is called so cannot call it in destructor where m_hWnd is NULL already.

    // Remove icon from system tray:
    NOTIFYICONDATA tnid; 
 
    tnid.cbSize = sizeof(NOTIFYICONDATA); 
    tnid.hWnd = GetSafeHwnd(); 
    tnid.uID = IDI_ICONSYSTRAY; 
         
    Shell_NotifyIcon(NIM_DELETE, &tnid); 
}


void CDTSpyView::FlushLog() {
    m_outFile.flush();
//    m_outFile<<"*********Flushing log!**********"<<endl;

}

void CDTSpyView::OnMynotifyicon(WPARAM wParam, LPARAM lParam) {
    UINT uID; 
    UINT uMouseMsg; 
 
    uID = (UINT) wParam; 
    uMouseMsg = (UINT) lParam; 
 
    if(uMouseMsg == WM_LBUTTONDOWN) {
        CPoint curPos;

#ifdef _WIN32_WCE
        curPos = CPoint(GetMessagePos());
#else
        GetCursorPos(&curPos);
#endif


        //ClientToScreen(&curPos);
        int xPos = curPos.x;  // horizontal position of cursor 
        int yPos = curPos.y;  // vertical position of cursor 

        CMenu menu;
        if(menu.LoadMenu(IDR_POPUPMENU)) {
            CMenu* pPopup = menu.GetSubMenu(0);
            ASSERT(pPopup != NULL);

            SetForegroundWindow();

#ifndef _WIN32_WCE
            ::TrackPopupMenu(pPopup->m_hMenu, 0, curPos.x, curPos.y, 0, 
                         GetSafeHwnd(), NULL);
#else
            pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
                xPos, yPos, this); // use main window for cmds
#endif

        }
        // BUGFIX: See "PRB: Menus for Notification Icons Don't Work Correctly"
//        pTargetWnd->PostMessage(WM_NULL, 0, 0);
        menu.DestroyMenu();
    }

    /*
        switch (uID) { 
            case IDI_MYBATTERYICON: 
 
                // The user clicked the battery icon. Display the 
                // battery status. 
                ShowBatteryStatus(); 
                break; 
 
            case IDI_MYPRINTERICON: 
 
                // The user clicked the printer icon. Display the 
                // status of the print job. 
                ShowJobStatus(); 
                break; 
 
            default: 
                break; 
        } 
     } 
     return; 
     */
} 



void CDTSpyView::OnAppExit() 
{
    PostQuitMessage(0);
    // This is how we stop ourselves.

    //    ::PostMessage(GetSafeHwnd(), WM_CLOSE, NULL, NULL);
    //    DestroyWindow( );
}

/** 
 * Safely copy a string. This function will allocate memory for dest.
 */

int CDTSpyView::SafeStrCopy(char ** dest, char * src, unsigned int max_len) {
    size_t len = strlen(src);

    if(len > max_len)
	    return -1;
    *dest = new char[len+1];
    strncpy(*dest, src, len);
    (*dest)[len] = '\0';
    return 0;
}
