// DTSpyView.h : interface of the CDTSpyView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DTSPYVIEW_H__AE09ED22_1DF5_43E9_B0ED_6440FBAD73FA__INCLUDED_)
#define AFX_DTSPYVIEW_H__AE09ED22_1DF5_43E9_B0ED_6440FBAD73FA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <list>
#include <string>
#include <map>
#include <fstream>
//#include "msword9.h"

// Forward class declaration:
class CAppInfo;
class CDocInfo;

using namespace std;

typedef map<string, CDocInfo *, less<string> > DocMap;
typedef map<int, CAppInfo *, less<int> > AppMap;
//typedef map<int, string, less<int> > DocMap;
typedef list<CAppInfo *> AppList;
typedef list<CDocInfo *> DocList;


/** 
 * Note for using Standard C++ library (e.g. list, map, string, etc):
 * 
 * First you must include the appropriate header. Note that these headers files
 * don't have .h suffix. I.e. to use list, just #include <list>.
 *
 * Then you have to specify std name space. You can do so by either 
 *
 *		using namespace std;
 * 
 * or add std:: to preceed class name (e.g. std::list). Note that Microsoft 
 * Visual C++ contains two versions of the C++ iostream class library and it 
 * may cause ambiguity if you use namespace std and #include <fstream.h>. The 
 * right thing is to use the STL and #include <fstream>.
 */

class CDocInfo {
public:
	CDocInfo() { elapsed = 0; pApp = NULL; }
	CDocInfo(const TCHAR *theTitle, const TCHAR *theUser, CAppInfo * theApp);
	void Write(ostream os);


	TCHAR title[MAX_PATH];
    TCHAR user[MAX_PATH];
	time_t elapsed;  // Time this doc is active, in seconds
	CAppInfo *pApp;
};

class CAppInfo {
public:
	CAppInfo() { ID = -1; nDocs = 0; needURL = false; }
	CAppInfo(const TCHAR *theCmd, int id, bool url = false);
	~CAppInfo();

	CDocInfo * AddDoc(const TCHAR *theTitle, const TCHAR *theUser);
	void Write(ostream os);
	
	TCHAR cmd[MAX_PATH];
	int ID;
	unsigned int nDocs; // Number of documents opened by this application.
	bool needURL;

	DocMap docMap;
};


#define TIME_OUT 10000           // Timer timeout. 1 sec.
#define TIME_FORCE_FLUSH 3600   // Force flushing log even when there is no document transit
#define TIME_NATURAL_FLUSH 60   // Flush log even when there is a document transit

// Define user message
#define WM_MYNOTIFYICON     (WM_USER+100)
#define WM_PROCESSDATA      (WM_USER+101)

#define BUFFERLEN 1024

class CDTSpyView : public CView
{
protected: // create from serialization only
	CDTSpyView();
	DECLARE_DYNCREATE(CDTSpyView)

// Attributes
public:
	CDTSpyDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDTSpyView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDTSpyView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	CString m_MSG;
	int m_cx, m_cy;
	CString m_fileName;
	ofstream m_outFile;

	// The following are used to process WM_COPYDATA message:
	TCHAR szBuffer[BUFFERLEN];
	int appID;
	int nCode;     // The event code 
	time_t now;


    UINT m_IDTimer;
    UINT m_nTicks;   
    // Number of timer timeouts. We will flush the log if enough ticks passed.
    
    HICON hIconSysTray;  // Icon to use on the systray

	CDocInfo * pDocPrev;    // Previously open doc
	time_t t_prev;          // When that doc is opened.

	AppMap appMap;

    BOOL ParseMessgeData(const TCHAR * szBuffer, TCHAR * szCmd, TCHAR * szDoc,
        TCHAR * szUser, bool url = false);
    void FlushLog();
    int SafeStrCopy(char ** dest, char * src, unsigned int max_len);
    void FixBackSlash(TCHAR * src, TCHAR * dest);

// Generated message map functions
protected:
	//{{AFX_MSG(CDTSpyView)
	afx_msg BOOL OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnAddapp();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
   	afx_msg void OnMynotifyicon(WPARAM wParam, LPARAM lParam);
	afx_msg void OnProcessdata(WPARAM wParam, LPARAM lParam);
	afx_msg void OnAppExit();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in DTSpyView.cpp
inline CDTSpyDoc* CDTSpyView::GetDocument()
   { return (CDTSpyDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DTSPYVIEW_H__AE09ED22_1DF5_43E9_B0ED_6440FBAD73FA__INCLUDED_)
